@@ -6,9 +6,6 @@
 # a = read.csv('~/Downloads/output.csv', stringsAsFactors = FALSE)
 # as.numeric(unlist(strsplit(a$pitch, ',')))
 
-# shinyBS needs to be included as a dependency (instead of just "import"):
-# see https://stackoverflow.com/questions/52649138/including-shinybs-in-a-package
-
 server = function(input, output, session) {
   # make overlaid plots resizable (js fix)
   shinyjs::js$inheritSize(parentDiv = 'specDiv')
@@ -62,6 +59,7 @@ server = function(input, output, session) {
     file.remove(paste0('www/', f))
   }
 
+  # reset the app
   reset = function() {
     if (myPars$print) print('Resetting...')
     myPars$pitch = NULL       # pitch contour
@@ -79,27 +77,29 @@ server = function(input, output, session) {
 
   resetSliders = function() {
     if (myPars$print) print('Resetting sliders...')
-    sliders_to_reset = names(input)[which(names(input) %in% rownames(defaults_analyze))]
+    sliders_to_reset = c(
+      names(input)[which(names(input) %in% names(pitch_app_defaults))],
+      'spec_ylim'
+    )
     for (v in sliders_to_reset) {
-      new_value = defaults_analyze[v, 'default']
-      try(updateSliderInput(session, v, value = new_value))
-      try(updateNumericInput(session, v, value = new_value))
-      updateSelectInput(session, 'wn', selected = 'gaussian')
-      updateCheckboxGroupInput(session, 'pitchMethods', selected = c('dom', 'autocor'))
-      updateCheckboxGroupInput(session, 'summaryFun', selected = c('mean', 'sd'))
-      updateTextInput(session, 'summaryFun_text', value = '')
-      updateSelectInput(session, 'pathfinding', selected = 'fast')
-      updateSliderInput(session, 'spec_ylim', value=c(0, defaults_analyze['spec_ylim','default']))
-      updateRadioButtons(session, 'spec_colorTheme', selected='bw')
-      updateSelectInput(session, 'osc', selected = 'linear')
+      new_value = pitch_app_defaults[[v]]
+      suppressWarnings(try(updateSliderInput(session, v, value = new_value)))
+      suppressWarnings(try(updateNumericInput(session, v, value = new_value)))
     }
+    updateSelectInput(session, 'wn', selected = pitch_app_defaults$wn)
+    updateCheckboxGroupInput(session, 'pitchMethods', selected = pitch_app_defaults$pitchMethods)
+    updateCheckboxGroupInput(session, 'summaryFun', selected = pitch_app_defaults$summaryFun)
+    updateTextInput(session, 'summaryFun_text', value = pitch_app_defaults$summaryFun_text)
+    updateSelectInput(session, 'pathfinding', selected = pitch_app_defaults$pathfinding)
+    updateRadioButtons(session, 'spec_colorTheme', selected = pitch_app_defaults$spec_colorTheme)
+    updateSelectInput(session, 'osc', selected = pitch_app_defaults$osc)
   }
-  observeEvent(input$reset_to_def, resetSliders())
-
+  observeEvent(input$reset_to_def, resetSliders(), ignoreNULL = FALSE)
 
   observeEvent(input$loadAudio, {
     if (myPars$print) print('Loading audio...')
-    reset()  # also triggers done()
+    done()
+    reset()
 
     # work only with audio files
     idx_audio = which(apply(matrix(input$loadAudio$type), 1, function(x) {
@@ -146,9 +146,12 @@ server = function(input, output, session) {
 
           frame_i = round(time_i / input$step)
           step_i = time_i[2] - time_i[1]
-          if (step_i != input$step) pitch_i = pitch_i[!duplicated(frame_i)]
+          if (step_i != input$step)
+            pitch_i = soundgen:::.resample(
+              audio = list(sound = pitch_i),
+              len = length(unique(frame_i)), lowPass = FALSE, na.rm = FALSE)
           manual_i = data.frame(
-            frame = frame_i, freq = pitch_i
+            frame = unique(frame_i), freq = pitch_i
           )
           myPars$history[[user_ann$file[i]]] = list(
             manual = manual_i[!is.na(manual_i$freq), ],
@@ -165,20 +168,6 @@ server = function(input, output, session) {
     # readAudio(1)  # read the first sound in queue
   })
 
-  observeEvent(input$showpanel, {
-    if(input$showpanel == TRUE) {
-      shinyjs::removeCssClass("Main", "col-sm-12")
-      shinyjs::addCssClass("Main", "col-sm-9")
-      shinyjs::show(id = "Sidebar")
-      shinyjs::enable(id = "Sidebar")
-    }
-    else {
-      shinyjs::removeCssClass("Main", "col-sm-9")
-      shinyjs::addCssClass("Main", "col-sm-12")
-      shinyjs::hide(id = "Sidebar")
-    }
-  })
-
   readAudio = function(i) {
     # reads an audio file with tuneR::readWave
     if (myPars$print) print('Reading audio...')
@@ -403,7 +392,6 @@ server = function(input, output, session) {
             y = as.numeric(rownames(myPars$spec_trimmed)),
             z = t(myPars$spec_trimmed),
             col = soundgen:::switchColorTheme(input$spec_colorTheme)(input$nColors),
-            log = '',
             yScale = 'linear',
             xlim = myPars$spec_xlim,
             xaxt = 'n',
@@ -417,7 +405,6 @@ server = function(input, output, session) {
           soundgen:::plotUnrasterized(
             myPars$reassigned,
             col = soundgen:::switchColorTheme(input$spec_colorTheme)(input$nColors),
-            log = '',
             yScale = 'linear',
             xlim = myPars$spec_xlim,
             xaxt = 'n',
@@ -1440,10 +1427,11 @@ server = function(input, output, session) {
   done = function() {
     # meaning we have finished editing pitch contour for a sound - prepares
     # the output
-    if (myPars$print) print('Running done()...')
+    if (myPars$print) print('Calling done()...')
     if (!is.null(myPars$myAudio_path) &&
-        !is.null(myPars$result) && !
-        is.null(myPars$pitch)) {
+        !is.null(myPars$result) &&
+        !is.null(myPars$pitch)) {
+      if (myPars$print) print('Running done()...')
       new = data.frame(
         file = basename(myPars$myAudio_filename),
         time = paste(round(myPars$X), collapse = ', '),
@@ -1495,9 +1483,11 @@ server = function(input, output, session) {
     if (!is.null(myPars$out)) {
       write.csv(myPars$out, 'www/temp.csv', row.names = FALSE)
       myPars$result$pitch = myPars$pitch
-      myPars$history[[myPars$myAudio_filename]]$detailed = myPars$result
-      myPars$history[[myPars$myAudio_filename]]$summary = myPars$out
-      myPars$history[[myPars$myAudio_filename]]$pitch = myPars$pitch
+      if (!is.null(myPars$myAudio_filename)) {
+        myPars$history[[myPars$myAudio_filename]]$detailed = myPars$result
+        myPars$history[[myPars$myAudio_filename]]$summary = myPars$out
+        myPars$history[[myPars$myAudio_filename]]$pitch = myPars$pitch
+      }
       my_pitch <<- myPars$history
     }
   }
@@ -1557,7 +1547,8 @@ server = function(input, output, session) {
     removeModal()
   })
   observeEvent(input$terminate_yes, {
-    stopApp(returnValue = myPars$history)
+    stopApp(returnValue = c(myPars$history, list(settings = reactiveValuesToList(input))))
+    rm(pitch_app_defaults, envir = .GlobalEnv)
   })
 
   observeEvent(input$about, {
@@ -1572,106 +1563,4 @@ server = function(input, output, session) {
       )
     }
   })
-
-  ## TOOLTIPS - have to be here instead of UI b/c otherwise problems with regulating delay
-  # (see https://stackoverflow.com/questions/47477237/delaying-and-expiring-a-shinybsbstooltip)
-  # STFT
-  shinyBS::addTooltip(session, id='reset_to_def', title = 'Reset all settings to default values', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='audioMethod', title = "Play audio with javascript (recommended in Firefox, doesn't work in Chrome) or with R (browser-independent, but then the cursor doesn't move, and you can't stop playback)", placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='windowLength', title = 'Length of STFT window, ms. Larger values improve frequency resolution at the expense of time resolution', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='step', title = 'Step between analysis frames, ms (alternative to "overlap")', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specType', title = 'Spectrogram type, argument "specType" in spectrogram(). Affects pitch tracking', placement="right", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  # shinyBS::addTooltip(session, id='overlap', title = 'Overlap between analysis frames, % (alternative to "step")', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='dynamicRange', title = 'Dynamic range of spectrogram, dB', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zp', title = 'Zero padding of STFT window (improves frequency resolution): 8 means 2^8 = 256, etc.', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='wn', title = 'Type of STFT window', placement="right", trigger="hover", options = tooltip_options)
-
-  # voicing
-  shinyBS::addTooltip(session, id='silence', title = 'Frames with RMS below silence threshold are not analyzed', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='entropyThres', title = 'Frames with Weiner entropy above entropy threshold are ignored when searching for pitch candidates', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='nCands', title = 'Maximum number of pitch candidates to use per method', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='minVoicedCands', title = 'Minimum number of pitch candidates that have to be defined to consider a frame voiced', placement="right", trigger="hover", options = tooltip_options)
-
-  # priors
-  shinyBS::addTooltip(session, id='pitchFloor', title = 'No candidates below this absolute threshold', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='pitchCeiling', title = 'No candidates above this absolute threshold', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='priorMean', title = 'Candidates close to this value are prioritized (how close is determined by priorSD)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='priorAdapt', title = 'Adds a second pass for finding the optimal pitch contour, with prior determined by the initial pitch estimates', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='priorSD', title = 'Determines the width of expected pitch range (standard deviation of gamma distribution around priorMean)', placement="right", trigger="hover", options = tooltip_options)
-
-  # trackers
-  shinyBS::addTooltip(session, id='domThres', title = 'Dominant frequency is defined as the lowest bin in a spectrum smoothed and normalized to range from 0 to 1 that it at least "domThres" high (1 = absolute maximum, ie peak frequency)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='domSmooth', title = 'Width of smoothing interval for finding the lowest dominant frequency band (low values = no smoothing)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='autocorThres', title = 'Voicing threshold for autocorrelation algorithm', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='autocorSmooth', title = 'Width of smoothing interval (in bins) for finding peaks in the autocorrelation function', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='autocorUpsample', title = 'Upsamples acf to this resolution (Hz) to improve accuracy in high frequencies', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='autocorBestPeak', title = 'Amplitude of the lowest best candidate relative to the absolute max of the acf', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='cepThres', title = 'Voicing threshold for cepstral algorithm', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='cepZp', title = 'Length of cepstral window after zero padding: 8 means 2^8 = 256, etc.', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specType', title = '"commonFactor" = greatest common factor of putative harmonics, "BaNa" = ratio of putative harmonics', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specThres', title = 'Voicing threshold for Ba-Na algorithm', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specPeak', title = 'Minimum amplitude of harmonics considered pitch candidates', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specHNRslope', title = '0 = same threshold regardless of HNR; positive = lower threshold in noisy sounds', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specSmooth', title = 'Width of window for detecting harmonics in the spectrum, Hz', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specMerge', title = 'Pitch candidates within specMerge semitones are merged with boosted certainty', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specSinglePeakCert', title = 'If pitch is calculated based on a single harmonic ratio (as opposed to several ratios converging on the same candidate), its certainty is taken to be specSinglePeakCert', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specRatios', title = 'For method = "commonFactor", the number of integer fractions to consider', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='hpsNum', title = 'How many times to downsample and then multiply the spectra', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='hpsThres', title = 'How high a spectral peak has to be to be considered a pitch candidate, ~0 to 1', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='hpsNorm', title = 'Rather arbitrary normalization of certainty in hps candidates intended to make them more comparable to other pitch tracking methods (0 = no boost in certainty, 2 = default quadratic)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='hpsPenalty', title = 'HPS performs worse at low frequencies (relative to windowLength), so low-frequency pitch candidates are penalized (0 = no penalization, ~10-20 = a lot)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zcThres', title = 'Minimum certainty in zero-crossing pitch candidates, ~0 to 1', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zcWin', title = 'Confidence in zero-crossing candidates depends on how stable the estimates are across zcWin glottal cycles', placement="right", trigger="hover", options = tooltip_options)
-
-  # pathfinder
-  shinyBS::addTooltip(session, id='summaryFun', title = "The function(s) used to summarize output", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='summaryFun_text', title = "If specified, overrides the options above. For short column names, define and name your function in R prior to starting pitch_app", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='automPathUpdate', title = "Update the optimal pitch contour automatically every time an anchor changes? Turn off to avoid delays when editing a long audio", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='pathfinding', title = "Method of finding the optimal path through pitch candidates: 'none' = best candidate per frame, 'fast' = simple heuristic, 'slow' = annealing (initial analysis only)", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='certWeight', title = 'Specifies how much we prioritize the certainty of pitch candidates vs. pitch jumps', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='shortestSyl', title = 'Shorter voiced segments (ms) will be treated as voiceless or merged with longer segments', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='shortestPause', title = "The smallest gap between voiced syllables (ms) that means they shouldn't be merged", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='smooth', title = 'Amount of median smoothing', placement="right", trigger="hover", options = tooltip_options)
-
-  # smoothing
-  shinyBS::addTooltip(session, id='interpolWin', title = "If no pitch candidates are found within ±interpolTol of the median 'best guess' over ±interpolWin, this median is added as an interpolated candidate", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='interpolTol', title = "Tolerated deviance from 'best guess' before adding an interpolated candidate: proportion of best guess frequency", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='interpolCert', title = "Certainty assigned to interpolated pitch candidates", placement="right", trigger="hover", options = tooltip_options)
-
-  # spectrogram
-  shinyBS::addTooltip(session, id='spec_ylim', title = "Range of displayed frequencies, kHz", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='spec_maxPoints', title = 'The number of points to plot in the spectrogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='spec_cex', title = "Magnification coefficient controlling the size of points showing pitch candidates", placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specContrast', title = 'Regulates the contrast of the spectrogram', placement="below", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='specBrightness', title = 'Regulates the brightness of the spectrogram', placement="below", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='nColors', title = 'The number of distinct colors in the spectrogram', placement="below", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='blur_freq', title = 'Gaussian filter of frequency: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-  shinyBS::addTooltip(session, id='blur_time', title = 'Gaussian filter of time: >0 = blur, <0 = unblur (sharpen)', placement="below", trigger="hover", options = list(delay = list(show = 1000, hide = 0)))
-
-  # oscillogram
-  shinyBS::addTooltip(session, id='osc', title = 'The type of oscillogram to show', placement="below", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='osc_height', title = 'The height of oscillogram, pixels', placement="below", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='osc_maxPoints', title = 'The number of points to plot in the oscillogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = tooltip_options)
-
-  # action buttons
-  shinyBS:::addTooltip(session, id='lastFile', title='Save and return to the previous file (PageUp)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS:::addTooltip(session, id='nextFile', title='Save and proceed to the next file (PageDown)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS:::addTooltip(session, id='selection_play', title='Play selection (SPACEBAR)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='selection_unvoice', title = 'Unvoice selection (U)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='selection_voice', title = 'Voice selection (V) - obviouslly, pitch estimates may be totally incorrect', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='selection_octaveUp', title = 'Raise pitch for selection by an octave (R)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='selection_octaveDown', title = 'Lower pitch for selection by an octave (L)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='selection_setPrior', title = 'Set a prior on expected pitch values corresponding to the selected frequency range (P)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='button_pathUpdate', title = 'Draw / refresh pitch contour (D) (only needed if "Out/Path/Update path automatically" is turned off)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='button_synth', title = 'Synthesize and play pitch contour', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='saveRes', title = 'Download results (see ?pitch_app for recovering unsaved data after a crash)', placement="right", trigger="hover", options = tooltip_options)
-
-  # navigation / zoom
-  shinyBS::addTooltip(session, id='zoomIn_freq', title = 'Zoom in frequency (+)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zoomOut_freq', title = 'Zoom out frequency (-)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='scrollLeft', title = 'Scroll left (arrow LEFT)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zoomOut', title = 'Zoom out time (arrow DOWN)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zoomToSel', title = 'Zoom to selection (S)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='zoomIn', title = 'Zoom in time (arrow UP)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS::addTooltip(session, id='scrollRight', title = 'Scroll right (arrow RIGHT)', placement="right", trigger="hover", options = tooltip_options)
 }
